%Algorithm to analyse the shape of a rocky coastline
%Written in GNU Octave 3.6.4
%Zuzanna M Swirad & W Gareth Rees
%z.m.swirad@durham.ac.uk
%Scott Polar Research Institute
%Cambridge University
%2014

clear
%Input to the algorithm: text file with a series of UTM coordinates. 
%1st column: longitude
%2nd column: latitude

%Uploading and preparing the data
data1=input('Enter the name of input file: ','s');
data2=load(data1);
r1=rows(data2);

%Calculation of segment and total length
data2(1,3)=0;
for rr1=2:r1
	data2(rr1,3)=sqrt((data2(rr1,1)-data2(rr1-1,1))^2+(data2(rr1,2)-data2(rr1-1,2))^2);
endfor

data2(:,4)=cumsum(data2(:,3));

len=data2(r1,4);
disp('The length of the coastline: ')
len

%Redrawing vertices to get them equally distributed along the coastline
meanlen=len/r1;
disp('The average distance between 2 vertices is: ')
meanlen

interp=input('Enter the distance value for the interpolation ');

%Creation of a new table [vertex information, headlands and embayment edges choice]
%Column 1: Distance from the beginning. Taking into account only interpolated vertices 
a=(0:interp:len)';

%Columns 2 and 3: longitude and latitude
a(:,2)=interp1(data2(:,4),data2(:,1),a(:,1),'linear');
a(:,3)=interp1(data2(:,4),data2(:,2),a(:,1),'linear');

r2=rows(a);

%Column 4: angle between vectors led between 2 nearest vertices. 
%Plot shows how noisy the angle values are along the coastline- they cannot be used for headland seeking.
a(1,4)=0;
a(r2,4)=0;

for rr2=2:(r2-1)
	ax=a(rr2-1,2)-a(rr2,2);
	ay=a(rr2-1,3)-a(rr2,3);
	bx=a(rr2+1,2)-a(rr2,2);
	by=a(rr2+1,3)-a(rr2,3);
	a(rr2,4)=atan2(ax*by-ay*bx, ax*bx+ay*by);
endfor

angle1=a(:,4)-pi*(sign(a(:,4)+1));
figure(1);
plot(a(:,1),angle1)
print('image1.png')
a(:,4)=angle1*180/pi+180;


%Column 5: angle between vectors led between vertices of given distance(step value m1)
%Step value n is a distance (1 is the nearest vertex etc) from given vertex 
%which will define which 2 vertices (on left and right side) will be used to lead vectors
%For first and last n vertices the cell in table will be filled with 0
%Plot shows that the noise seen in previous picture is not present any more- angle values change gradually
n=input('Enter the step value for vector drawing ');

a(1:n,5)=0;
a((r2-n):r2,5)=0;

for rr3=(n+1):(r2-n);
	ax=a(rr3-n,2)-a(rr3,2);
	ay=a(rr3-n,3)-a(rr3,3);
	bx=a(rr3+n,2)-a(rr3,2);
	by=a(rr3+n,3)-a(rr3,3);
	a(rr3,5)=atan2(ax*by-ay*bx, ax*bx+ay*by);
endfor

angle2=a(:,5)-pi*(sign(a(:,5)+1));
figure(2);
plot(a(:,5),angle2)
print('image2.png')
a(:,5)=angle2*180/pi+180;

%Column 6: Choice of headlands (i.e. vertices at which the angle between 2 adjacent vectors [defined with step value m1]
%is larger than adequate angles for 8 (4 and 4) neighbouring vertices; m2=4)
%It gets value 1 if the angle from previous column fulfils this criterion.
a(1:n,6)=0;
a((r2-n):r2,6)=0;

for rr3=(n+1):(r2-n);
	if (angle2(rr3)>angle2(rr3-1));
	if (angle2(rr3)>angle2(rr3+1));
	if (angle2(rr3)>angle2(rr3-2));
	if (angle2(rr3)>angle2(rr3+2));
	if (angle2(rr3)>angle2(rr3-3));
	if (angle2(rr3)>angle2(rr3+3));
	if (angle2(rr3)>angle2(rr3-4));
	if (angle2(rr3)>angle2(rr3+4));
		a(rr3,6)=1;
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
		else
		a(rr3,6)=0;
	endif
endfor

s1=sum(a(:,6));
disp('Number of headlands: ')
s1

figure(3);
plot (a(:,2),a(:,3))
hold on

for rr4=1:r2
	if a(rr4,6)==1
		plot (a(rr4,2),a(rr4,3),'@11')
	endif
endfor

print('image3.png')

%Column 7: Choice of embayment edges: vertices in vicinity (max distance = 8 vertices; m3=8) of the formerly chosen
%headlands, at which the angle between 2 shortest vectors (from Column 4) gets value which makes it replace headland.
%6 possible outcomes of the procedure:
%1. Headland is the only boundary between 2 embayments
%2. One of the vertices on the left side of the headland is the only boundary between 2 embayments
%3. One of the vertices on the right side of the headland is the only boundary between 2 embayments
%4. One of the vertices on the left side of the headland is one of embayment edges and the headland is the second
%5. Headland is one of embayment edges and one of the vertices on the right side of the headland is the second
%6. One of the vertices on the left side of the headland is one of embayment edges and one of the vertices on the right side of the headland is the second

%A vertex starts to be an embayment edge if its angle value is larger than 206* 
%while headland angle value is smaller than 200* and if its angle is larger than 228* 
%regardless headland angle value.
%A headland becomes an embayment edge if none of the vertices on the left and/or right became
%embayment edges or when its angle value is larger than 240* regardless other vertices angles. 

a(1:r2,7)=0;

t1=206;
t2=228;
t3=200;
t4=240;

for rr4=1:r2
	if a(rr4,6)==1
		max1=max(a((rr4-8):(rr4-1),4));
			for g1=(rr4-8):(rr4-1)
				if a(g1,4)==max1
					if a(g1,4)>t1
						if a(rr4,4)<t3
							a(g1,7)=1;
						endif
					endif
					if a(g1,4)>t2
						a(g1,7)=1;	
					endif
				endif
			endfor
		max2=max(a((rr4+1):(rr4+8),4));
			for g2=(rr4+1):(rr4+8)
				if a(g2,4)==max2
					if a(g2,4)>t1
						if a(rr4,4)<t3
							a(g2,7)=1;
						endif
					endif	
					if a(g2,4)>t2
						a(g2,7)=1;		
					endif
				endif
			endfor	
		summa=sum (a((rr4-8):(rr4+8),7));
			if summa==0
				if a(rr4,4)>180
					a(rr4,7)=1;
				endif
			endif
		if a(rr4,4)>t4
			a(rr4,7)=1;
		endif	
	endif	
endfor

s2=sum(a(:,7));
disp('Number of embayment edges: ')
s2

%Plot: distribution of embayment edges along the coastline
figure(4);
plot (a(:,2),a(:,3))
hold on

for rr4=1:r2
	if a(rr4,7)==1
		plot (a(rr4,2),a(rr4,3),'@11')
	endif
endfor

print('image4.png')

%Columns 8-9: Segment and cumulative length of coastline after interpolation
a(1,8)=0;
for rr5=2:r2
	a(rr5,8)=sqrt((a(rr5,2)-a(rr5-1,2))^2+(a(rr5,3)-a(rr5-1,3))^2);
endfor

a(:,9)=cumsum(a(:,8));

name1=input('Enter name for the table with vertex information: ','s');
dlmwrite(name1,a,' ');

%Creation of a new table [embayment morphometry]
%Column 1: Number of row which contains embayment edge
counter=1;
for rr4=1:r2
	if(a(rr4,7)==1)
		v(counter,1)=rr4;
		counter=counter+1;
	endif
endfor

%Column 2: Embayment width [meters] (length of a vector between given and former vertex)
v(1,2)=0;
for ss=2:s2
	v(ss,2)=sqrt((a(v(ss),2)-a(v(ss-1),2))^2+(a(v(ss),3)-a(v(ss-1),3))^2);
endfor

%Column 3: Embayment orientation [degrees] (angle perpendicular to the width vector facing directed 
%outside the embayment); i.e. direction which an embayment faces
v(1,3)=0;

for ss=2:s2
	v(ss,3)=atan2(a(v(ss),2)-a(v(ss-1),2), a(v(ss),3)-a(v(ss-1),3));
	v(ss,3)=v(ss,3)*180/pi+90;
	if v(ss,3)<0
		v(ss,3)=v(ss,3)+360;
	endif
endfor

%Column 4: Embayment's area [square meters]
for ss=2:s2
	area1=0;
	i1=v(ss-1);
	i2=v(ss);
	for i3=i1:(i2-1)
		x1=a(i3,2);
		x2=a(i3+1,2);
		y1=a(i3,3);
		y2=a(i3+1,3);
		area1=area1+(x2-x1)*(y2+y1)/2;
	endfor
	x1=a(i2,2);
	x2=a(i1,2);
	y1=a(i2,3);
	y2=a(i1,3);
	area1=area1+(x2-x1)*(y2+y1)/2;
	v(ss,4)=area1;
endfor

%Column 5: Embayment's  mean depth [meters] (area/width)
v(:,5)=v(:,4)./v(:,2);

%Column 6: Coefficient of shape (area/area of a circle whose diameter equals embayment's width)
v(:,6)=(pi*(v(:,2)./2).^2)/2;
v(:,6)=v(:,4)./v(:,6);

%Column 7: Coastline length [meters] (sum of segment's length for each embayment)
lennew=a(v(:,1),9);
v(1,7)=0;

for ss=2:s2
	v(ss,7)=lennew(ss)-lennew(ss-1);
endfor

%Column 8: sinuosity index (width/coastline length)
v(:,8)=v(:,7)./v(:,2);

%Column 9: Coefficient of compactness (coastline length/length of a semicyrcle with the same area)
v(:,9)=pi*(v(:,2)/2);
v(:,9)=v(:,7)./v(:,9);

%Creation of a multi-plot to compare sizes and shapes of embayments (rotated to face South).
%Used to calculate further parameters.
%Column 10: Embayment's maximum depth [meters] (detection of the vertex with the highest value on y axis)
%Column 11: Embayment's left area [square meters] (area limited by vectors between vertices and a straight 
%perpendicular to the width which divides the width in the middle)

v(1,10)=0;
v(1,11)=0;

for ss=2:s2
	nvv=v(ss,1)-v(ss-1,1)+1;
	vv3(1,3)=0;
	vv3(1,4)=0;
	theta=v(ss,3)-180;
	tr=theta*pi/180;
	m=[cos(tr), -sin(tr); sin(tr), cos(tr)];
	vv1=[a(v(ss-1,1):v(ss,1),2), a(v(ss-1,1):v(ss,1),3)];
	vv3=(m*vv1')';
	for s3=2:nvv
		vv3(s3,3)=vv3(s3,1)-vv3(1,1);
		vv3(s3,4)=vv3(s3,2)-vv3(1,2);
		figure(5);
		subplot (6,5,ss-1)
		plot(vv3(1:nvv,3),vv3(1:nvv,4), [0, 1200], [-100, 400])
	endfor
	v(ss,10)=max(vv3(:,4));
	srod3=vv3(nvv,3)/2;
	counter=1;
	for s4=1:nvv
		if (vv3(s4,3)<srod3);
			lewa(counter,1)=s4;
			counter=counter+1;
		endif
	endfor
	srod4=(srod3*vv3(counter,4)-srod3*vv3(counter-1,4)-vv3(counter-1,3)*vv3(counter,4)+vv3(counter-1,4)*vv3(counter,3))/(vv3(counter,3)-vv3(counter-1,3));
	for s5=1:(counter-1)
		lewa(s5,1)=vv3(s5,3);
		lewa(s5,2)=vv3(s5,4);
	endfor
	lewa(counter,1)=srod3;
	lewa(counter,2)=srod4;
	lewa(counter+1,1)=srod3;
	lewa(counter+1,2)=0;
	area=0;
		for i3=1:counter
			x1=lewa(i3,1);
			x2=lewa(i3+1,1);
			y1=lewa(i3,2);
			y2=lewa(i3+1,2);
			area=area+(x2-x1)*(y2+y1)/2;
		endfor
		v(ss,11)=area;
endfor

print('image5.png')

%Column 12: Embayment's right area [square meters] (total area-left area)
v(:,12)=v(:,4)-v(:,11);

%Column 13: Coefficient of asymmetry (left area/right area)
v(:,13)=v(:,11)./v(:,12);

%Column 14: Form factor (max depth/width)
v(:,14)=v(:,10)./v(:,2);

%So far all sections between embayment edges were analysed.
%However, in cases when a headland was split into 2 embayment edges, between them there was 
%a headland with more complex morphology or larger peninsula or random section of shoreline 
%which should not be included in embayment list. 
%Removing these sections from embayment list consists on removing records with non-positive area and/or 
%sinuosity index lower than 1.03 


v(:,15)=0;
for ss1=1:s2
	if v(ss1,4)<=0;
		v(ss1,15)=1;
		else
			if v(ss1,8)<1.03;
			v(ss1,15)=1;
			endif
	endif
endfor 

s3=rows(v)-sum(v(:,15))
disp('Number of embayments: ')
s3
	
for ss2=s2:-1:1
	if v(ss2,15)==1;
		v(ss2,:)=[];
	endif
endfor

%Rearrangement of columns in order to give logical output table
%Columns contain: 1. Number; 2. Width; 3. Orientation; 4. Area; 5. Left area; 6. Right area,
%7. Coefficient of asymmetry; 8. Mean depth; 9. Max depth; 10. Form factor; 11. Coefficient of shape;
%12. Shoreline length; 13. Sinuosity index; 14. Coefficient of compactness 
r3=rows(v);
v2(:,1)=1:r3;
v2(:,2)=v(:,2);
v2(:,3)=v(:,3);
v2(:,4)=v(:,4);
v2(:,5)=v(:,11);
v2(:,6)=v(:,12);
v2(:,7)=v(:,13);
v2(:,8)=v(:,5);
v2(:,9)=v(:,10);
v2(:,10)=v(:,14);
v2(:,11)=v(:,6);
v2(:,12)=v(:,7);
v2(:,13)=v(:,8);
v2(:,14)=v(:,9);

name2=input('Enter name for the embayment parameter table: ','s');
dlmwrite(name2,v2,' ');